package javax.swing.plaf.nimbus;

import java.beans.PropertyChangeListener;
import javax.swing.plaf.UIResource;
import java.util.Iterator;
import java.util.HashMap;
import java.awt.LayoutManager;
import java.awt.Container;
import java.awt.BorderLayout;
import javax.swing.JToolBar;
import javax.swing.plaf.ColorUIResource;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import sun.swing.ImageIconUIResource;
import javax.swing.GrayFilter;
import sun.swing.plaf.synth.SynthIcon;
import javax.swing.Icon;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.beans.PropertyChangeEvent;
import javax.swing.plaf.BorderUIResource;
import sun.swing.plaf.WindowsKeybindings;
import sun.swing.plaf.GTKKeybindings;
import javax.swing.UIManager;
import javax.swing.plaf.synth.SynthStyle;
import javax.swing.plaf.synth.Region;
import javax.swing.JComponent;
import javax.swing.plaf.synth.SynthStyleFactory;
import java.util.Map;
import javax.swing.UIDefaults;
import javax.swing.plaf.synth.SynthLookAndFeel;

public class NimbusLookAndFeel extends SynthLookAndFeel
{
    private static final String[] COMPONENT_KEYS;
    private NimbusDefaults defaults;
    private UIDefaults uiDefaults;
    private NimbusLookAndFeel.DefaultsListener defaultsListener;
    private Map<String, Map<String, Object>> compiledDefaults;
    private boolean defaultListenerAdded;
    
    public NimbusLookAndFeel() {
        this.defaultsListener = new NimbusLookAndFeel.DefaultsListener();
        this.compiledDefaults = null;
        this.defaultListenerAdded = false;
        this.defaults = new NimbusDefaults();
    }
    
    @Override
    public void initialize() {
        super.initialize();
        this.defaults.initialize();
        SynthLookAndFeel.setStyleFactory(new SynthStyleFactory() {
            @Override
            public SynthStyle getStyle(JComponent component, Region region) {
                return NimbusLookAndFeel.this.defaults.getStyle(component, region);
            }
        });
    }
    
    @Override
    public void uninitialize() {
        super.uninitialize();
        this.defaults.uninitialize();
        ImageCache.getInstance().flush();
        UIManager.getDefaults().removePropertyChangeListener(this.defaultsListener);
    }
    
    @Override
    public UIDefaults getDefaults() {
        if (this.uiDefaults == null) {
            String systemProperty = this.getSystemProperty("os.name");
            int n = systemProperty != null && systemProperty.contains("Windows");
            this.uiDefaults = super.getDefaults();
            this.defaults.initializeDefaults(this.uiDefaults);
            if (!(n == 0)) {
                WindowsKeybindings.installKeybindings(this.uiDefaults);
            }
            else {
                GTKKeybindings.installKeybindings(this.uiDefaults);
            }
            this.uiDefaults.put("TitledBorder.titlePosition", Integer.valueOf(1));
            this.uiDefaults.put("TitledBorder.border", new BorderUIResource(new LoweredBorder()));
            this.uiDefaults.put("TitledBorder.titleColor", this.getDerivedColor("text", 0.0f, 0.0f, 0.23f, 0, true));
            this.uiDefaults.put("TitledBorder.font", new NimbusDefaults.DerivedFont("defaultFont", 1.0f, Boolean.valueOf(true), null));
            this.uiDefaults.put("OptionPane.isYesLast", Boolean.valueOf((boolean)(n == 0)));
            this.uiDefaults.put("Table.scrollPaneCornerComponent", new UIDefaults.ActiveValue() {
                @Override
                public Object createValue(UIDefaults uiDefaults) {
                    return new TableScrollPaneCorner();
                }
            });
            this.uiDefaults.put("ToolBarSeparator[Enabled].backgroundPainter", new ToolBarSeparatorPainter());
            for (String str : NimbusLookAndFeel.COMPONENT_KEYS) {
                String string = str + ".foreground";
                if (!this.uiDefaults.containsKey(string)) {
                    this.uiDefaults.put(string, new NimbusLookAndFeel.NimbusProperty(str, "textForeground"));
                }
                String string2 = str + ".background";
                if (!this.uiDefaults.containsKey(string2)) {
                    this.uiDefaults.put(string2, new NimbusLookAndFeel.NimbusProperty(str, "background"));
                }
                String string3 = str + ".font";
                if (!this.uiDefaults.containsKey(string3)) {
                    this.uiDefaults.put(string3, new NimbusLookAndFeel.NimbusProperty(str, "font"));
                }
                String string4 = str + ".disabledText";
                if (!this.uiDefaults.containsKey(string4)) {
                    this.uiDefaults.put(string4, new NimbusLookAndFeel.NimbusProperty(str, "Disabled", "textForeground"));
                }
                String string5 = str + ".disabled";
                if (!this.uiDefaults.containsKey(string5)) {
                    this.uiDefaults.put(string5, new NimbusLookAndFeel.NimbusProperty(str, "Disabled", "background"));
                }
            }
            this.uiDefaults.put("FileView.computerIcon", new NimbusLookAndFeel.LinkProperty("FileChooser.homeFolderIcon"));
            this.uiDefaults.put("FileView.directoryIcon", new NimbusLookAndFeel.LinkProperty("FileChooser.directoryIcon"));
            this.uiDefaults.put("FileView.fileIcon", new NimbusLookAndFeel.LinkProperty("FileChooser.fileIcon"));
            this.uiDefaults.put("FileView.floppyDriveIcon", new NimbusLookAndFeel.LinkProperty("FileChooser.floppyDriveIcon"));
            this.uiDefaults.put("FileView.hardDriveIcon", new NimbusLookAndFeel.LinkProperty("FileChooser.hardDriveIcon"));
        }
        return this.uiDefaults;
    }
    
    public static NimbusStyle getStyle(JComponent c, Region region) {
        return (NimbusStyle)SynthLookAndFeel.getStyle(c, region);
    }
    
    @Override
    public String getName() {
        return "Nimbus";
    }
    
    @Override
    public String getID() {
        return "Nimbus";
    }
    
    @Override
    public String getDescription() {
        return "Nimbus Look and Feel";
    }
    
    @Override
    public boolean shouldUpdateStyleOnAncestorChanged() {
        return true;
    }
    
    @Override
    protected boolean shouldUpdateStyleOnEvent(PropertyChangeEvent ev) {
        String propertyName = ev.getPropertyName();
        if ("name" == propertyName || "ancestor" == propertyName || "Nimbus.Overrides" == propertyName || "Nimbus.Overrides.InheritDefaults" == propertyName || "JComponent.sizeVariant" == propertyName) {
            this.defaults.clearOverridesCache((JComponent)ev.getSource());
            return true;
        }
        return super.shouldUpdateStyleOnEvent(ev);
    }
    
    public void register(Region region, String s) {
        this.defaults.register(region, s);
    }
    
    private String getSystemProperty(String theProp) {
        return (String)AccessController.doPrivileged(new GetPropertyAction(theProp));
    }
    
    @Override
    public Icon getDisabledIcon(JComponent component, Icon icon) {
        if ((icon instanceof SynthIcon)) {
            SynthIcon synthIcon = (SynthIcon)icon;
            BufferedImage compatibleTranslucentImage = EffectUtils.createCompatibleTranslucentImage(synthIcon.getIconWidth(), synthIcon.getIconHeight());
            Graphics2D graphics = compatibleTranslucentImage.createGraphics();
            synthIcon.paintIcon(component, graphics, 0, 0);
            graphics.dispose();
            return new ImageIconUIResource(GrayFilter.createDisabledImage(compatibleTranslucentImage));
        }
        return super.getDisabledIcon(component, icon);
    }
    
    public Color getDerivedColor(String s, float n, float n2, float n3, int n4, boolean b) {
        return this.defaults.getDerivedColor(s, n, n2, n3, n4, b);
    }
    
    protected final Color getDerivedColor(Color color, Color color2, float n, boolean b) {
        int deriveARGB = deriveARGB(color, color2, n);
        if (b) {
            return new ColorUIResource(deriveARGB);
        }
        return new Color(deriveARGB);
    }
    
    protected final Color getDerivedColor(Color color, Color color2, float n) {
        return this.getDerivedColor(color, color2, n, true);
    }
    
    static Object resolveToolbarConstraint(JToolBar comp) {
        if (comp != null) {
            Container parent = comp.getParent();
            if (parent != null) {
                LayoutManager layout = parent.getLayout();
                if ((layout instanceof BorderLayout)) {
                    Object constraints = ((BorderLayout)layout).getConstraints(comp);
                    if (constraints == "South" || constraints == "East" || constraints == "West") {
                        return constraints;
                    }
                    return "North";
                }
            }
        }
        return "North";
    }
    
    static int deriveARGB(Color color, Color color2, float n) {
        return (color.getAlpha() + Math.round((float)(color2.getAlpha() - color.getAlpha()) * n) & 0xFF) << 24 | (color.getRed() + Math.round((float)(color2.getRed() - color.getRed()) * n) & 0xFF) << 16 | (color.getGreen() + Math.round((float)(color2.getGreen() - color.getGreen()) * n) & 0xFF) << 8 | (color.getBlue() + Math.round((float)(color2.getBlue() - color.getBlue()) * n) & 0xFF);
    }
    
    static String parsePrefix(String s) {
        if (s == null) {
            return null;
        }
        boolean n = false;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 == '\"') {
                n = (n == 0);
            }
            else if ((char1 == '[' || char1 == '.') && n == 0) {
                return s.substring(0, i);
            }
        }
        return null;
    }
    
    Map<String, Object> getDefaultsForPrefix(String s) {
        if (this.compiledDefaults == null) {
            this.compiledDefaults = new HashMap();
            for (Map.Entry entry : UIManager.getDefaults().entrySet()) {
                if ((entry.getKey() instanceof String)) {
                    this.addDefault((String)entry.getKey(), entry.getValue());
                }
                else {
                    continue;
                }
            }
            if (!this.defaultListenerAdded) {
                UIManager.getDefaults().addPropertyChangeListener(this.defaultsListener);
                this.defaultListenerAdded = true;
            }
        }
        return (Map)this.compiledDefaults.get(s);
    }
    
    private void addDefault(String s, Object o) {
        if (this.compiledDefaults == null) {
            return;
        }
        String prefix = parsePrefix(s);
        if (prefix != null) {
            Map<String, Object> map = (Map)this.compiledDefaults.get(prefix);
            if (map == null) {
                map = new HashMap();
                this.compiledDefaults.put(prefix, map);
            }
            map.put(s, o);
        }
        return;
    }
    
    static void access$700(NimbusLookAndFeel nimbusLookAndFeel, String s, Object o) {
        nimbusLookAndFeel.addDefault(s, o);
    }
    
    static {
        COMPONENT_KEYS = new String[] { "ArrowButton", "Button", "CheckBox", "CheckBoxMenuItem", "ColorChooser", "ComboBox", "DesktopPane", "DesktopIcon", "EditorPane", "FileChooser", "FormattedTextField", "InternalFrame", "InternalFrameTitlePane", "Label", "List", "Menu", "MenuBar", "MenuItem", "OptionPane", "Panel", "PasswordField", "PopupMenu", "PopupMenuSeparator", "ProgressBar", "RadioButton", "RadioButtonMenuItem", "RootPane", "ScrollBar", "ScrollBarTrack", "ScrollBarThumb", "ScrollPane", "Separator", "Slider", "SliderTrack", "SliderThumb", "Spinner", "SplitPane", "TabbedPane", "Table", "TableHeader", "TextArea", "TextField", "TextPane", "ToggleButton", "ToolBar", "ToolTip", "Tree", "Viewport" };
    }
    
    private class LinkProperty implements UIDefaults.ActiveValue, UIResource
    {
        private String dstPropName;
        
        private LinkProperty(String dstPropName) {
            this.dstPropName = dstPropName;
        }
        
        @Override
        public Object createValue(UIDefaults uiDefaults) {
            return UIManager.get(this.dstPropName);
        }
    }
    
    private class NimbusProperty implements UIDefaults.ActiveValue, UIResource
    {
        private String prefix;
        private String state;
        private String suffix;
        private boolean isFont;
        
        private NimbusProperty(String prefix, String s) {
            this.state = null;
            this.prefix = prefix;
            this.suffix = s;
            this.isFont = "font".equals(s);
        }
        
        private NimbusProperty(NimbusLookAndFeel nimbusLookAndFeel, String s, String state, String s2) {
            this(nimbusLookAndFeel, s, s2);
            this.state = state;
        }
        
        @Override
        public Object createValue(UIDefaults uiDefaults) {
            Object o = null;
            if (this.state != null) {
                o = NimbusLookAndFeel.this.uiDefaults.get(this.prefix + "[" + this.state + "]." + this.suffix);
            }
            if (o == null) {
                o = NimbusLookAndFeel.this.uiDefaults.get(this.prefix + "[Enabled]." + this.suffix);
            }
            if (o == null) {
                if (this.isFont) {
                    o = NimbusLookAndFeel.this.uiDefaults.get("defaultFont");
                }
                else {
                    o = NimbusLookAndFeel.this.uiDefaults.get(this.suffix);
                }
            }
            return o;
        }
    }
    
    private class DefaultsListener implements PropertyChangeListener
    {
        @Override
        public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
            String propertyName = propertyChangeEvent.getPropertyName();
            if ("UIDefaults".equals(propertyName)) {
                NimbusLookAndFeel.this.compiledDefaults = null;
            }
            else {
                NimbusLookAndFeel.access$700(NimbusLookAndFeel.this, propertyName, propertyChangeEvent.getNewValue());
            }
        }
    }
}
